function love.load()
  --macros
  KEY = "return"
  DURATION = 5--seconds

  do--buttonWaiter prototype/class; an object to monitor/wait a keypress and runs the corresponding callback function if the key is pressed during its wait.
    buttonWaiter = {}
    --create an instance of the buttonWaiter prototype
    function buttonWaiter:init(pKey, pDuration, pFunction, ...)
      local prototype = {
        --parameters
        key = pKey;
        duration = pDuration;
        callback = pFunction;
        parameters = {...};
        --internal variables
        downCounter = pDuration;
        isActive = false;
      }
      return setmetatable(prototype, {__index = self})
    end
    --starts/stops waiting the button
    function buttonWaiter:activate(pBool)
      self.isActive = pBool
    end
    --updates the internal timer, checks if it reaches 0, execute the callback function with its parameters
    function buttonWaiter:update(dt)
      if self.isActive then
        self.downCounter = self.downCounter - dt
        if self.downCounter < 0 then
          --reset timer, deactivate self
          self.downCounter = self.duration
          self.isActive = false
        end
      end
    end
    function buttonWaiter:keypressed(key)
      --activate function if key is pressed and waiter is still active (timer hasn't reached 0)
      if key == self.key and self.isActive then
        self.callback(unpack(self.parameters))
      end
    end
  end
  
  do--example of callback function for the buttonWaiter
    --upvalues or "private" variables
    local doShout = false
    local text = ""
    local shoutCount = 0
    --this callback function toggles the flag doShout that controls whether you want to print the text or not
    function shout(pText)
      doShout = not doShout
      text = pText
      if not doShout then shoutCount = shoutCount + 1 end
    end
    --prints the text if the doShout flag is set
    function printShout(x, y)
      if doShout then
        love.graphics.setColor(255, 150, 100)
        love.graphics.print(text, x, y)
        love.graphics.setColor(255, 255, 255)
      end
      love.graphics.print(string.format('you shouted "%s" %d times in %d seconds.', text, shoutCount, DURATION), x, y + 15)
    end
  end
  
  --create new instance of buttonWaiter, called bw1, that would call shout("HURRAY!") if the KEY key is pressed in DURATION seconds after this instance is activated
  bw1 = buttonWaiter:init(KEY, DURATION, shout, "HURRAY!")
  bw1:activate(true)--activate the buttonWaiter instance (start counting!)
end

function love.update(dt)
  bw1:update(dt)
end

function love.draw()
  love.graphics.print(string.format("you can still bash [%s] in %.1f s!", KEY, (bw1.isActive and bw1.downCounter or 0)), 30, 30)
  printShout(30, 60)
  love.graphics.print("[F5] restart", 30, 105)
  love.graphics.print("[esc] quit", 30, 120)
end

function love.keypressed(key)
  if key == "escape" or key == "q" then
    love.event.push("q")
  elseif key == "f5" then
    love.filesystem.load("main.lua")()
    love.load()
  end
  bw1:keypressed(key)
end
